##############################################################################
#
# This script is used construct the network plots for a dataset. 
#
##############################################################################
# Required libraries: --------------------------------------------------------
library(igraph)

# Script parameters: ---------------------------------------------------------
file_name <- "" # Input the .csv file name containing the count data.
                # The columns should correspond to genes, and rows to samples.
                # All entries are expected to be counts.
counts <- read.csv(file_name)
results <- analyze_individual_network(counts, make_plots = FALSE)
n_edges <- 39 # Number of edges to include in the graph. If `sorted_scores` 
              # contains ties, there may a greater number of edges displayed.


# Analysis of network. --------------------------------------------------------
load(paste(dir, scores_file_name, sep = ""))
results$adj_matrix <- NULL # Remove to save memory.
results$scores <- abs(results$scores)
gc()

# Sort genes by degree.
degree <- apply(results$likelihoods <= results$threshold, 2, sum)
n_genes <- ncol(results$likelihoods)
order_by_degree <- order(degree, decreasing = TRUE)[1:n_genes]
results$threshold <- NULL # Remove to save memory.
gc()

n_top <- 10
cat("Top", n_top, "genes by degree:\n", 
    paste(1:n_top, ". ", results$genes[order_by_degree[1:n_top]], 
          "(", degree[order_by_degree[1:n_top]], ")\n", sep = ""), 
    "\n")

# Sort genes by average score.
mean_score <- apply(results$scores, 2, mean)
order_by_score <- order(mean_score, decreasing = TRUE)

n_top <- 10
cat("Top", n_top, "genes by average score:\n", 
    paste(1:n_top, ". ", results$genes[order_by_score[1:n_top]], 
          " (", mean_score[order_by_score[1:n_top]], ")\n", sep = ""), 
    "\n")

# Graph the network; subset using the n_edges heaviest (highest scores) edges.
sorted_scores <- sort(results$scores[lower.tri(results$scores)], 
                      decreasing = TRUE)[1:10000]
# There will likely be ties amoung the sorted scores. A good choice of n_edges
# may need to be done by inspection.
min_score <- sorted_scores[n_edges]
subset_genes <- apply(results$scores, 2, function(x) any(x >= min_score))
small_nw <- results$scores[subset_genes, subset_genes]
colnames(small_nw) <- results$genes[subset_genes]

g <- graph_from_adjacency_matrix(small_nw, 
                                 mode = "undirected",
                                 weighted = TRUE)
V(g)$label <- results$genes[subset_genes]
V(g)$size <- sapply(round(sqrt(degree[subset_genes])), max, 0) 
V(g)$frame.color <- "white"
g <- subgraph.edges(g, which(E(g)$weight >= min_score))
E(g)$width <- ((E(g)$weight - min(E(g)$weight)) / 
                 (max(E(g)$weight) - min(E(g)$weight)) * 4 + 0.5) / 2.5

plot(g, vertex.color = "orange", vertex.label.font = 2, 
     vertex.label.color = "blue", vertex.label.cex = 0.7, 
     edge.color = "gray", layout = layout_nicely,
     main = nw)